import socket, logging, platform, os, subprocess, time, sys, argparse, threading
from ttproto.core.data  import *
from ttproto.core.lib.all   import *

logger = logging.getLogger(__name__)

"""
PRECONDITIONS:
- Have a running CoAP server at 127.0.0.1:5683, with the /resurces configured in TD_COAP ETSI doc.
- have tcpdump installed
"""

IP='127.0.0.1'
UDP_PORT=5683

def _launch_sniffer(filename, filter_if = None):
    logger.info('Launching packet capture..')

    if (filter_if is None) or (filter_if == ''):
        sys_type = platform.system()
        if sys_type == 'Darwin':
            filter_if = 'lo0'
        else:
            filter_if = 'lo'
    logger.info('logging in %s' % filter_if)
            # TODO windows?

    # lets try to remove the filemame in case there's a previous execution of the TC
    try:
        params = 'rm ' + filename
        os.system(params)
    except:
        pass

    params = 'tcpdump  -K -i ' + filter_if + ' -s 200 ' + ' -U -w ' + filename + ' udp -vv ' + '&'
    os.system(params)
    logger.info('creating process tcpdump with: %s' % params)
    # TODO we need to catch tcpdump: <<tun0: No such device exists>> from stderr

    return True


def _stop_sniffer():
    proc = subprocess.Popen(["pkill", "-INT", "tcpdump"], stdout=subprocess.PIPE)
    proc.wait()
    logger.info('Packet capture stopped')
    return True

def default_PASS_client_emulator(type, code, mid, token, payload ,option):
    coap_msg =  CoAP(type=type, code=code, mid=mid, tok=token, pl=payload, opt=option)
    #coap_msg = CoAP(type='con', code='get')
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP,UDP_PORT))
    #receive message
    reply = sock.recv(1024)
    #translate message
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)
    sock.close()

# def server_emulator():
#     # waiting for receive a message CoAP/UDP/IPv4
#     sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
#     sock.bind((IP, UDP_PORT))
#     msg_recv, addr = sock.recvfrom(1024)
#     #translate message
#     binary_msg = Message(msg_recv, CoAP)
#     coap_message = binary_msg.get_value()
#     return coap_message, addr


def td_coap_core_01_PASS_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client', CoAPOptionList([CoAPOptionUriPath ("test_option_client")]))


def td_coap_core_01_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test"),CoAPOptionContentFormat(3)] ))
    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

#fail : no opt CoAPOptionContentFormat(3)
def td_coap_core_01_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()


def td_coap_core_02_PASS_client_emulator():
    default_PASS_client_emulator('con', 'delete', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client")]))

def td_coap_core_02_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.02, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test"),CoAPOptionContentFormat(3)] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_02_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.02, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_03_PASS_client_emulator():
    default_PASS_client_emulator('con', 'put', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"),CoAPOptionContentFormat(3)]))

def td_coap_core_03_FAIL_client_emulator():
    # fail : no opt CoAPOptionContentFormat(3)
    default_PASS_client_emulator('con', 'put', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client")]))

def td_coap_core_03_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_03_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04)

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_03_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_04_PASS_client_emulator():
    default_PASS_client_emulator('con', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"),CoAPOptionContentFormat(3)]))

def td_coap_core_04_FAIL_client_emulator():
    # fail : no opt CoAPOptionContentFormat(3)
    default_PASS_client_emulator('con', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client")]))

def td_coap_core_04_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_04_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04)

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_04_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_05_PASS_client_emulator():
    default_PASS_client_emulator('non', 'get', 9, 1, 'test payload client', CoAPOptionList([CoAPOptionUriPath("test_option_client"),CoAPOptionContentFormat(3)]))

def td_coap_core_05_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_05_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_06_PASS_client_emulator():
    default_PASS_client_emulator('non', 'delete', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_06_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.02, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_06_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.02)

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_06_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.02, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_07_PASS_client_emulator():
    default_PASS_client_emulator('non', 'put', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_07_FAIL_NO_PL_client_emulator():
    default_PASS_client_emulator('non', 'put', 9, 1, '',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_07_FAIL_NO_OPT_client_emulator():
    default_PASS_client_emulator('non', 'put', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client")]))

def td_coap_core_07_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_07_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04)

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_07_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_08_PASS_client_emulator():
    default_PASS_client_emulator('non', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_08_FAIL_NO_PL_client_emulator():
    default_PASS_client_emulator('non', 'post', 9, 1, '',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_08_FAIL_NO_OPT_client_emulator():
    default_PASS_client_emulator('non', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client")]))

def td_coap_core_08_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_08_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04)

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_08_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.04, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_09_PASS_client_emulator():
    coap_msg =  CoAP(type='con', code='get', mid=9, tok=1, pl='test payload client', opt=CoAPOptionList([CoAPOptionUriPath ("test option client"),CoAPOptionContentFormat(3)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP,UDP_PORT))

    # receive acknowledgment
    ack_recev = sock.recv(1024)
    # translate acknowledgment
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    #receive message
    reply = sock.recv(1024)
    #translate message
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    sock.close()

    return reply

def td_coap_core_09_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    #build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_09_FAIL_NO_OPT_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    #build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    #build response
    # fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server")]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_09_FAIL_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    #build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    #build response
    # fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_09_FAIL_NO_TOK_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    #build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    #build response
    # fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], code=2.05, pl='test payload server', opt=CoAPOptionList([CoAPOptionUriPath ("test option server"),CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_10_PASS_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_10_FAIL_TOK_0_BYTE_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, b'', 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_10_FAIL_TOK_9_BYTE_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, b'0123456879', 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_10_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_10_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    #fail : no opt CoAPOptionContentFormat(3)
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test_option_server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()


def td_coap_core_11_PASS_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=1, pl='test payload client',
                    opt=CoAPOptionList([CoAPOptionUriPath("test option client"), CoAPOptionContentFormat(3)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive acknowledgment
    ack_recev = sock.recv(1024)
    # translate acknowledgment
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    # receive message
    reply = sock.recv(1024)
    # translate message
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    sock.close()

    return reply

def td_coap_core_11_FAIL_TOK_0_BYTE_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'', pl='test payload client',
                    opt=CoAPOptionList([CoAPOptionUriPath("test option client"), CoAPOptionContentFormat(3)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive acknowledgment
    ack_recev = sock.recv(1024)
    # translate acknowledgment
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    # receive message
    reply = sock.recv(1024)
    # translate message
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    sock.close()

    return reply

def td_coap_core_11_FAIL_TOK_9_BYTE_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'0123456879', pl='test payload client',
                    opt=CoAPOptionList([CoAPOptionUriPath("test option client"), CoAPOptionContentFormat(3)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive acknowledgment
    ack_recev = sock.recv(1024)
    # translate acknowledgment
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    # receive message
    reply = sock.recv(1024)
    # translate message
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    sock.close()

    return reply

def td_coap_core_11_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    # build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server"), CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    # send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_11_FAIL_NO_TOK_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    # build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server"), CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    # send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()


def td_coap_core_11_FAIL_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    # build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server"), CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    # send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))

    # receive acknowledgment
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate acknowledgment
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    sock.close()

def td_coap_core_12_PASS_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, b'', 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_12_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_12_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=1, code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_13_PASS_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("seg1"),CoAPOptionUriPath("seg2"), CoAPOptionUriPath("seg3"), CoAPOptionContentFormat(3)]))

def td_coap_core_13_FAIL_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath("/seg1"),CoAPOptionUriPath("/seg2"), CoAPOptionUriPath("/seg3"), CoAPOptionContentFormat(3)]))

def td_coap_core_13_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_14_PASS_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriQuery("first=1"),CoAPOptionUriQuery("second=2"), CoAPOptionContentFormat(3)]))

def td_coap_core_14_FAIL_client_emulator():
    default_PASS_client_emulator('con', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriQuery("first=1"), CoAPOptionContentFormat(3)]))


def td_coap_core_14_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_14_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server")] ))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_17_PASS_client_emulator():
    default_PASS_client_emulator('non', 'get', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath ("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_17_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_17_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server', opt= CoAPOptionList([CoAPOptionUriPath ("test_option_server")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    time.sleep(2)
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_17_FAIL_ACK_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build ack
    ack = CoAP(type='ack', mid=coap_message['mid'], code=0)
    print('***** SERVER send acknowledgment : *****')
    print(ack)
    # send acknowledgment
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (addr[0], addr[1]))

    time.sleep(2)
    # build response
    coap_response = CoAP(type='non', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05, pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionUriPath("test option server"), CoAPOptionContentFormat(3)]))
    print('***** SERVER What we send : *****')
    print(coap_response)
    # send response
    msg, msg_bytes = coap_response.build_message()
    sock.sendto(msg_bytes, (addr[0], addr[1]))
    sock.close()

def td_coap_core_18_PASS_client_emulator():
    default_PASS_client_emulator('con', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath ("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_18_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationPath ("location1"),CoAPOptionLocationPath ("location2"),CoAPOptionLocationPath ("location3"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_18_FAIL_2_location_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationPath ("location1"),CoAPOptionLocationPath ("location2"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_18_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationPath (".."),CoAPOptionLocationPath ("/location2"), CoAPOptionLocationPath ("/location3"), CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_19_PASS_client_emulator():
    default_PASS_client_emulator('con', 'post', 9, 1, 'test payload client',
                                 CoAPOptionList([CoAPOptionUriPath ("test_option_client"), CoAPOptionContentFormat(3)]))

def td_coap_core_19_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationQuery("first=1"),CoAPOptionLocationQuery("second=2"),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_19_PASS_NO_PL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, opt= CoAPOptionList([CoAPOptionLocationQuery("first=1"),CoAPOptionLocationQuery("second=2")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_19_FAIL_server_emulator():
    #no coapoptionformt with payload
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationQuery("first=1"),CoAPOptionLocationQuery("second=2")]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_19_FAIL_WRONG_OPT_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv, addr = connexion.recvfrom(1024)
    #translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)
    # print(addr[0])
    # print(addr[1])
    # print(coap_message['mid'])
    #build response
    coap_response = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.01, pl='test payload server', opt= CoAPOptionList([CoAPOptionLocationQuery("?=."),CoAPOptionLocationQuery("&=.."),CoAPOptionContentFormat(3)]))

    print('***** SERVER What we send : *****')
    print(coap_response)
    #send response
    msg, msg_bytes = coap_response.build_message()
    connexion.sendto(msg_bytes, (addr[0], addr[1]))
    connexion.close()

def td_coap_core_20_PASS_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'01',
                    opt=CoAPOptionList([CoAPOptionAccept(0)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive message1
    ack_recev = sock.recv(1024)
    # translate message1
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    # build response
    ack = CoAP(type='con', mid=8, tok=2, code='get', opt=CoAPOptionList([CoAPOptionAccept(41)]))
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send response
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    # receive message2
    reply = sock.recv(1024)
    # translate message2
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    sock.close()

    return reply

def td_coap_core_20_FAIL_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'01',
                    opt=CoAPOptionList([CoAPOptionAccept(0)]))
    print('***** CLIENT What we send : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive message1
    ack_recev = sock.recv(1024)
    # translate message1
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT acknowledgment receive : *****')
    print(coap_ack_recv)

    # build response
    ack = CoAP(type='con', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], code='get', opt=CoAPOptionList([CoAPOptionAccept(41)]))
    print('***** CLIENT send acknowledgment : *****')
    print(ack)
    # send response
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    # receive message2
    reply = sock.recv(1024)
    # translate message2
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive : *****')
    print(coap_message)

    sock.close()

    return reply

def td_coap_core_20_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build response1
    coap_response1 = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionAccept(0), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send : *****')
    print(coap_response1)
    # send response
    msg, msg_bytes1 = coap_response1.build_message()
    sock.sendto(msg_bytes1, (addr[0], addr[1]))

    # receive response
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate response
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    # build response
    coap_response2 = CoAP(type='con', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionAccept(41), CoAPOptionContentFormat(41)]))
    print('***** SERVER What we send : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    sock.sendto(msg_bytes2, (addr[0], addr[1]))
    sock.close()

def td_coap_core_20_FAIL_WRONG_FORMAT_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive : *****')
    print(coap_message)

    # build response1
    coap_response1 = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionAccept(0), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send : *****')
    print(coap_response1)
    # send response
    msg, msg_bytes1 = coap_response1.build_message()
    sock.sendto(msg_bytes1, (addr[0], addr[1]))

    # receive response
    msg_ack_recv, addr = sock.recvfrom(1024)
    # translate response
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER acknowledgment receive : *****')
    print(coap_ack_recv)

    # build response
    coap_response2 = CoAP(type='con', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionAccept(41), CoAPOptionContentFormat(4)]))
    print('***** SERVER What we send : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    sock.sendto(msg_bytes2, (addr[0], addr[1]))
    sock.close()

def td_coap_core_21_PASS_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'01', opt=CoAPOptionList([ CoAPOptionUriPath(val='validate')]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive message1
    ack_recev = sock.recv(1024)
    # translate message1
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_ack_recv)

    # build response
    ack = CoAP(type='con', code='get', mid=10, tok=b'02',  opt=coap_ack_recv['opt'])
    print('***** CLIENT What we send 2 : *****')
    print(ack)
    # send response
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    # receive message2
    reply = sock.recv(1024)
    # translate message2
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message)

    # build response
    ack3 = CoAP(type='con', code='put')
    print('***** CLIENT What we send 3 : *****')
    print(ack3)
    # send response
    msg, msg_bytes_ack3 = ack3.build_message()
    sock.sendto(msg_bytes_ack3, (IP, UDP_PORT))

    # receive message
    reply3 = sock.recv(1024)
    # translate message2
    binary_msg3 = Message(reply3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** CLIENT What we receive 3 : *****')
    print(coap_message3)

     # build response
    ack4 = CoAP(type='con', code='get', mid=11, tok=b'03', opt=coap_message['opt'])
    print('***** CLIENT What we send 4 : *****')
    print(ack4)
    # send response
    msg, msg_bytes_ack4 = ack4.build_message()
    sock.sendto(msg_bytes_ack4, (IP, UDP_PORT))

    # receive message2
    reply4 = sock.recv(1024)
    # translate message2
    binary_msg4 = Message(reply4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** CLIENT What we receive 4 : *****')
    print(coap_message4)

    sock.close()

    return reply

def td_coap_core_21_FAIL_client_emulator():
    coap_msg = CoAP(type='con', code='get', mid=9, tok=b'02', opt=CoAPOptionList([ CoAPOptionUriPath(val='validate')]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg)
    msg, msg_bytes = coap_msg.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes, (IP, UDP_PORT))

    # receive message1
    ack_recev = sock.recv(1024)
    # translate message1
    binary_msg_ack = Message(ack_recev, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_ack_recv)

    # build response
    ack = CoAP(type='con', code='get', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'],  opt=coap_ack_recv['opt'])
    print('***** CLIENT What we send 2 : *****')
    print(ack)
    # send response
    msg, msg_bytes_ack = ack.build_message()
    sock.sendto(msg_bytes_ack, (IP, UDP_PORT))

    # receive message2
    reply = sock.recv(1024)
    # translate message2
    binary_msg = Message(reply, CoAP)
    coap_message = binary_msg.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message)

    # build response
    ack3 = CoAP(type='con', code='put')
    print('***** CLIENT What we send 3 : *****')
    print(ack3)
    # send response
    msg, msg_bytes_ack3 = ack3.build_message()
    sock.sendto(msg_bytes_ack3, (IP, UDP_PORT))

    # receive message
    reply3 = sock.recv(1024)
    # translate message2
    binary_msg3 = Message(reply3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** CLIENT What we receive 3 : *****')
    print(coap_message3)

     # build response
    ack4 = CoAP(type='con', code='get', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], opt=coap_message['opt'])
    print('***** CLIENT What we send 4 : *****')
    print(ack4)
    # send response
    msg, msg_bytes_ack4 = ack4.build_message()
    sock.sendto(msg_bytes_ack4, (IP, UDP_PORT))

    # receive message2
    reply4 = sock.recv(1024)
    # translate message2
    binary_msg4 = Message(reply4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** CLIENT What we receive 4 : *****')
    print(coap_message4)

    sock.close()

    return reply

def td_coap_core_21_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr1 = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message)

    # build response1
    coap_response1 = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionETag(1), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    # send response
    msg, msg_bytes1 = coap_response1.build_message()
    sock.sendto(msg_bytes1, (addr1[0], addr1[1]))

    # receive response
    msg_ack_recv, addr2 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_ack_recv)

    # build response
    coap_response2 = CoAP(type='con', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], code=2.03,
                         opt=coap_ack_recv['opt'])
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    sock.sendto(msg_bytes2, (addr2[0], addr2[1]))

    # receive response
    msg_ack_recv3, addr3 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack3 = Message(msg_ack_recv3, CoAP)
    coap_ack_recv3 = binary_msg_ack3.get_value()
    print('***** SERVER What we receive 3 : *****')
    print(coap_ack_recv3)

    # build response
    coap_response3 = CoAP(type='con', mid=coap_ack_recv3['mid'], tok=coap_ack_recv3['tok'], code=2.04,
                          opt=coap_ack_recv3['opt'])
    print('***** SERVER What we send 3 : *****')
    print(coap_response3)
    # send response
    msg, msg_bytes3 = coap_response3.build_message()
    sock.sendto(msg_bytes3, (addr3[0], addr3[1]))

    # receive response
    msg_ack_recv4, addr4 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack4 = Message(msg_ack_recv4, CoAP)
    coap_ack_recv4 = binary_msg_ack4.get_value()
    print('***** SERVER What we receive 4 : *****')
    print(coap_ack_recv4)

    # build response
    coap_response4 = CoAP(type='con', mid=coap_ack_recv4['mid'], tok=coap_ack_recv4['tok'], code=2.05, pl='test payload server3',
                          opt=CoAPOptionList([CoAPOptionETag(3), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 4 : *****')
    print(coap_response4)
    # send response
    msg, msg_bytes4 = coap_response4.build_message()
    sock.sendto(msg_bytes4, (addr4[0], addr4[1]))
    sock.close()

def td_coap_core_21_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((IP, UDP_PORT))
    msg_recv, addr1 = sock.recvfrom(1024)
    # translate message
    binary_msg = Message(msg_recv, CoAP)
    coap_message = binary_msg.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message)

    # build response1
    coap_response1 = CoAP(type='con', mid=coap_message['mid'], tok=coap_message['tok'], code=2.05,
                         pl='test payload server',
                         opt=CoAPOptionList([CoAPOptionETag(1), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    # send response
    msg, msg_bytes1 = coap_response1.build_message()
    sock.sendto(msg_bytes1, (addr1[0], addr1[1]))

    # receive response
    msg_ack_recv, addr2 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack = Message(msg_ack_recv, CoAP)
    coap_ack_recv = binary_msg_ack.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_ack_recv)

    # build response
    coap_response2 = CoAP(type='con', mid=coap_ack_recv['mid'], tok=coap_ack_recv['tok'], code=2.03,
                         opt=coap_ack_recv['opt'])
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    sock.sendto(msg_bytes2, (addr2[0], addr2[1]))

    # receive response
    msg_ack_recv3, addr3 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack3 = Message(msg_ack_recv3, CoAP)
    coap_ack_recv3 = binary_msg_ack3.get_value()
    print('***** SERVER What we receive 3 : *****')
    print(coap_ack_recv3)

    # build response
    coap_response3 = CoAP(type='con', mid=coap_ack_recv3['mid'], tok=coap_ack_recv3['tok'], code=2.04,
                          opt=coap_ack_recv3['opt'])
    print('***** SERVER What we send 3 : *****')
    print(coap_response3)
    # send response
    msg, msg_bytes3 = coap_response3.build_message()
    sock.sendto(msg_bytes3, (addr3[0], addr3[1]))

    # receive response
    msg_ack_recv4, addr4 = sock.recvfrom(1024)
    # translate response
    binary_msg_ack4 = Message(msg_ack_recv4, CoAP)
    coap_ack_recv4 = binary_msg_ack4.get_value()
    print('***** SERVER What we receive 4 : *****')
    print(coap_ack_recv4)

    # build response
    coap_response4 = CoAP(type='con', mid=coap_ack_recv4['mid'], tok=coap_ack_recv4['tok'], code=2.05, pl='test payload server',
                          opt=CoAPOptionList([CoAPOptionETag(3), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 4 : *****')
    print(coap_response4)
    # send response
    msg, msg_bytes4 = coap_response4.build_message()
    sock.sendto(msg_bytes4, (addr4[0], addr4[1]))
    sock.close()

def td_coap_core_22_PASS_client_emulator():
    coap_msg1 = CoAP(type='con', code='get', mid=9, tok=b'02', opt=CoAPOptionList([ CoAPOptionUriPath(val='validate')]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg1)
    msg, msg_bytes1 = coap_msg1.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes1, (IP,UDP_PORT))
    #receive message
    reply1 = sock.recv(1024)
    #translate message
    binary_msg1 = Message(reply1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_message1)

    ETAG1=coap_message1['opt']
    coap_msg2 = CoAP(type='con', code='put', mid=10, tok=b'03',pl='test payload client2',
                          opt=CoAPOptionList(
                              [ CoAPOptionIfMatch(ETAG1[0]['val']), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 2 : *****')
    print(coap_msg1)
    msg, msg_bytes2 = coap_msg2.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes2, (IP, UDP_PORT))
    # receive message
    reply2 = sock.recv(1024)
    # translate message
    binary_msg2 = Message(reply2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message2)

    coap_msg3 = CoAP(type='con', code='get', mid=11, tok=b'04', pl='test payload client2',
                     opt=CoAPOptionList(
                         [CoAPOptionUriPath(val='validate'),CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 3 : *****')
    print(coap_msg3)
    msg, msg_bytes3 = coap_msg3.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes3, (IP, UDP_PORT))
    # receive message
    reply3 = sock.recv(1024)
    # translate message
    binary_msg3 = Message(reply3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** CLIENT What we receive 3 : *****')
    print(coap_message3)

    coap_msg4 = CoAP(type='con', code='put', pl='test payload client4',
                     opt=CoAPOptionList([CoAPOptionUriPath(val='validate'),CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 4 : *****')
    print(coap_msg4)
    msg, msg_bytes4 = coap_msg4.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes4, (IP, UDP_PORT))
    # receive message
    reply4 = sock.recv(1024)
    # translate message
    binary_msg4 = Message(reply4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** CLIENT What we receive 4 : *****')
    print(coap_message4)

    ETAG2 = coap_message3['opt']
    coap_msg5 = CoAP(type='con', code='put', mid=12, tok=b'05',pl='test payload client5',
                          opt=CoAPOptionList(
                              [ CoAPOptionIfMatch(ETAG2[0]['val']), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** CLIENT What we send 5 : *****')
    print(coap_msg5)
    msg, msg_bytes5 = coap_msg5.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes5, (IP, UDP_PORT))
    # receive message
    reply5 = sock.recv(1024)
    # translate message
    binary_msg5 = Message(reply5, CoAP)
    coap_message5 = binary_msg5.get_value()
    print('***** CLIENT What we receive 5 : *****')
    print(coap_message5)

    sock.close()

def td_coap_core_22_FAIL_client_emulator():
    coap_msg1 = CoAP(type='con', code='get', mid=9, tok=b'02', opt=CoAPOptionList([ CoAPOptionUriPath(val='validate')]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg1)
    msg, msg_bytes1 = coap_msg1.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes1, (IP,UDP_PORT))
    #receive message
    reply1 = sock.recv(1024)
    #translate message
    binary_msg1 = Message(reply1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_message1)

    ETAG1=coap_message1['opt']
    coap_msg2 = CoAP(type='con', code='put', mid=9, tok=b'02',pl='test payload client2',
                          opt=CoAPOptionList(
                              [ CoAPOptionIfMatch(ETAG1[0]['val']), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 2 : *****')
    print(coap_msg1)
    msg, msg_bytes2 = coap_msg2.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes2, (IP, UDP_PORT))
    # receive message
    reply2 = sock.recv(1024)
    # translate message
    binary_msg2 = Message(reply2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message2)

    coap_msg3 = CoAP(type='con', code='get', mid=9, tok=b'02', pl='test payload client2',
                     opt=CoAPOptionList(
                         [CoAPOptionUriPath(val='validate'),CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 3 : *****')
    print(coap_msg3)
    msg, msg_bytes3 = coap_msg3.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes3, (IP, UDP_PORT))
    # receive message
    reply3 = sock.recv(1024)
    # translate message
    binary_msg3 = Message(reply3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** CLIENT What we receive 3 : *****')
    print(coap_message3)

    coap_msg4 = CoAP(type='con', code='put', pl='test payload client4',
                     opt=CoAPOptionList([CoAPOptionUriPath(val='validate'),CoAPOptionContentFormat(0)]))

    print('***** CLIENT What we send 4 : *****')
    print(coap_msg4)
    msg, msg_bytes4 = coap_msg4.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes4, (IP, UDP_PORT))
    # receive message
    reply4 = sock.recv(1024)
    # translate message
    binary_msg4 = Message(reply4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** CLIENT What we receive 4 : *****')
    print(coap_message4)

    ETAG2 = coap_message3['opt']
    coap_msg5 = CoAP(type='con', code='put', mid=9, tok=b'02',pl='test payload client5',
                          opt=CoAPOptionList(
                              [ CoAPOptionIfMatch(ETAG2[0]['val']), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** CLIENT What we send 5 : *****')
    print(coap_msg5)
    msg, msg_bytes5 = coap_msg5.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes5, (IP, UDP_PORT))
    # receive message
    reply5 = sock.recv(1024)
    # translate message
    binary_msg5 = Message(reply5, CoAP)
    coap_message5 = binary_msg5.get_value()
    print('***** CLIENT What we receive 5 : *****')
    print(coap_message5)

    sock.close()

def td_coap_core_22_PASS_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type='con', mid=coap_message1['mid'], tok=coap_message1['tok'], code=2.05,
                          pl='test payload server',
                          opt=CoAPOptionList(
                              [CoAPOptionETag(1), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    #receive response
    msg_recv2, addr2 = connexion.recvfrom(1024)
    # translate message
    binary_msg2 = Message(msg_recv2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_message2)
    # build response
    coap_response2 = CoAP(type='con', mid=coap_message2['mid'], tok=coap_message2['tok'], code=2.04,
                          pl='test payload server',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    connexion.sendto(msg_bytes2, (addr2[0], addr2[1]))

    # receive response
    msg_recv3, addr3 = connexion.recvfrom(1024)
    # translate message
    binary_msg3 = Message(msg_recv3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** SERVER What we receive 3 : *****')
    print(coap_message3)
    # build response
    coap_response3 = CoAP(type='con', mid=coap_message3['mid'], tok=coap_message3['tok'], code=2.05,
                          pl=coap_message2['pl'],
                          opt=CoAPOptionList([CoAPOptionETag(2), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 3 : *****')
    print(coap_response3)
    # send response
    msg, msg_bytes3 = coap_response3.build_message()
    connexion.sendto(msg_bytes3, (addr3[0], addr3[1]))

    # receive response
    msg_recv4, addr4 = connexion.recvfrom(1024)
    # translate message
    binary_msg4 = Message(msg_recv4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** SERVER What we receive 4 : *****')
    print(coap_message4)
    # build response
    coap_response4 = CoAP(type='con', mid=coap_message4['mid'], tok=coap_message4['tok'], code=2.04,
                          pl='test payload server3',
                          opt=CoAPOptionList(
                              [CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 4 : *****')
    print(coap_response4)
    # send response
    msg, msg_bytes4 = coap_response4.build_message()
    connexion.sendto(msg_bytes4, (addr4[0], addr4[1]))

    # receive response
    msg_recv5, addr5 = connexion.recvfrom(1024)
    # translate message
    binary_msg5 = Message(msg_recv5, CoAP)
    coap_message5 = binary_msg5.get_value()
    print('***** SERVER What we receive 5 : *****')
    print(coap_message5)
    # build response
    coap_response5 = CoAP(type='con', mid=coap_message5['mid'], tok=coap_message5['tok'], code=4.12,
                          pl='test payload server3',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 5 : *****')
    print(coap_response5)
    # send response
    msg, msg_bytes5 = coap_response5.build_message()
    connexion.sendto(msg_bytes5, (addr5[0], addr5[1]))

    connexion.close()

def td_coap_core_22_FAIL_server_emulator():
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type='con', mid=coap_message1['mid'], tok=coap_message1['tok'], code=2.05,
                          pl='test payload server',
                          opt=CoAPOptionList(
                              [CoAPOptionETag(1), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    #receive response
    msg_recv2, addr2 = connexion.recvfrom(1024)
    # translate message
    binary_msg2 = Message(msg_recv2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_message2)
    # build response
    coap_response2 = CoAP(type='con', mid=coap_message2['mid'], tok=coap_message2['tok'], code=2.04,
                          pl='test payload server',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    connexion.sendto(msg_bytes2, (addr2[0], addr2[1]))

    # receive response
    msg_recv3, addr3 = connexion.recvfrom(1024)
    # translate message
    binary_msg3 = Message(msg_recv3, CoAP)
    coap_message3 = binary_msg3.get_value()
    print('***** SERVER What we receive 3 : *****')
    print(coap_message3)
    # build response
    coap_response3 = CoAP(type='con', mid=coap_message3['mid'], tok=coap_message3['tok'], code=2.05,
                          pl=coap_message2['pl'],
                          opt=CoAPOptionList([CoAPOptionETag(2), CoAPOptionUriPath(val='validate'), CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 3 : *****')
    print(coap_response3)
    # send response
    msg, msg_bytes3 = coap_response3.build_message()
    connexion.sendto(msg_bytes3, (addr3[0], addr3[1]))

    # receive response
    msg_recv4, addr4 = connexion.recvfrom(1024)
    # translate message
    binary_msg4 = Message(msg_recv4, CoAP)
    coap_message4 = binary_msg4.get_value()
    print('***** SERVER What we receive 4 : *****')
    print(coap_message4)
    # build response
    coap_response4 = CoAP(type='con', mid=coap_message4['mid'], tok=coap_message4['tok'], code=2.04,
                          pl='test payload server3',
                          opt=CoAPOptionList(
                              [CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 4 : *****')
    print(coap_response4)
    # send response
    msg, msg_bytes4 = coap_response4.build_message()
    connexion.sendto(msg_bytes4, (addr4[0], addr4[1]))

    # receive response
    msg_recv5, addr5 = connexion.recvfrom(1024)
    # translate message
    binary_msg5 = Message(msg_recv5, CoAP)
    coap_message5 = binary_msg5.get_value()
    print('***** SERVER What we receive 5 : *****')
    print(coap_message5)
    # build response
    coap_response5 = CoAP(type='con', mid=36, tok=b'36', code=4.12,
                          pl='test payload server3',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 5 : *****')
    print(coap_response5)
    # send response
    msg, msg_bytes5 = coap_response5.build_message()
    connexion.sendto(msg_bytes5, (addr5[0], addr5[1]))

    connexion.close()

def td_coap_core_23_PASS_client_emulator():
    #DONT WORK
    coap_msg1 = CoAP(type='con', code='put', mid=9, tok=b'02',pl='test payload client1', opt=CoAPOptionList([CoAPOptionIfNoneMatch(), CoAPOptionUriPath(val='create1'), CoAPOptionContentFormat(0)]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg1)
    msg, msg_bytes1 = coap_msg1.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes1, (IP,UDP_PORT))
    #receive message
    reply1 = sock.recv(1024)
    #translate message
    binary_msg1 = Message(reply1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_message1)

    coap_msg2 = CoAP(type='con', code='put', mid=10, tok=b'03',pl='test payload client2',
                          opt=CoAPOptionList(
                              [CoAPOptionContentFormat(0), CoAPOptionUriPath(val='create1'), CoAPOptionIfNoneMatch()]))

    print('***** CLIENT What we send 2 : *****')
    print(coap_msg1)
    msg, msg_bytes2 = coap_msg2.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes2, (IP, UDP_PORT))
    # receive message
    reply2 = sock.recv(1024)
    # translate message
    binary_msg2 = Message(reply2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message2)


    sock.close()

def td_coap_core_23_FAIL_client_emulator():
    #DONT WORK
    coap_msg1 = CoAP(type='con', code='put', mid=9, tok=b'02',pl='test payload client1', opt=CoAPOptionList([CoAPOptionIfNoneMatch(), CoAPOptionUriPath(val='create1'), CoAPOptionContentFormat(0)]))
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg1)
    msg, msg_bytes1 = coap_msg1.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes1, (IP,UDP_PORT))
    #receive message
    reply1 = sock.recv(1024)
    #translate message
    binary_msg1 = Message(reply1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_message1)

    coap_msg2 = CoAP(type='con', code='put', mid=9, tok=b'02',pl='test payload client2',
                          opt=CoAPOptionList(
                              [CoAPOptionContentFormat(0), CoAPOptionUriPath(val='create1'), CoAPOptionIfNoneMatch()]))

    print('***** CLIENT What we send 2 : *****')
    print(coap_msg1)
    msg, msg_bytes2 = coap_msg2.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.sendto(msg_bytes2, (IP, UDP_PORT))
    # receive message
    reply2 = sock.recv(1024)
    # translate message
    binary_msg2 = Message(reply2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** CLIENT What we receive 2 : *****')
    print(coap_message2)


    sock.close()

def td_coap_core_23_PASS_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type='con', mid=coap_message1['mid'], tok=coap_message1['tok'], code=2.01,
                          pl='test payload server',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    #receive response
    msg_recv2, addr2 = connexion.recvfrom(1024)
    # translate message
    binary_msg2 = Message(msg_recv2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_message2)
    # build response
    coap_response2 = CoAP(type='con', mid=coap_message2['mid'], tok=coap_message2['tok'], code=4.12,
                          pl='')
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    connexion.sendto(msg_bytes2, (addr2[0], addr2[1]))

    connexion.close()

def td_coap_core_23_FAIL_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type='con', mid=coap_message1['mid'], tok=coap_message1['tok'], code=2.01,
                          pl='test payload server',
                          opt=CoAPOptionList([CoAPOptionContentFormat(0)]))
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    #receive response
    msg_recv2, addr2 = connexion.recvfrom(1024)
    # translate message
    binary_msg2 = Message(msg_recv2, CoAP)
    coap_message2 = binary_msg2.get_value()
    print('***** SERVER What we receive 2 : *****')
    print(coap_message2)
    # build response
    coap_response2 = CoAP(type='con', mid=36, tok=b'36', code=4.1,
                          pl='')
    print('***** SERVER What we send 2 : *****')
    print(coap_response2)
    # send response
    msg, msg_bytes2 = coap_response2.build_message()
    connexion.sendto(msg_bytes2, (addr2[0], addr2[1]))

    connexion.close()

def td_coap_core_31_PASS_client_emulator():
    #DONT WORK
    coap_msg1 = CoAP(type='con', code=0, mid=9, tok=0)
    print('***** CLIENT What we send 1 : *****')
    print(coap_msg1)
    msg, msg_bytes1 = coap_msg1.build_message()
    # build and send CoAP/UDP/IPv4 message
    sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    sock.sendto(msg_bytes1, (IP,UDP_PORT))
    #receive message
    reply1 = sock.recv(1024)
    #translate message
    binary_msg1 = Message(reply1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** CLIENT What we receive 1 : *****')
    print(coap_message1)

    sock.close()

def td_coap_core_31_PASS_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type=3, mid=coap_message1['mid'], tok=coap_message1['tok'], code=0)
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    connexion.close()

def td_coap_core_31_FAIL_PL_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type=3, mid=coap_message1['mid'], tok=coap_message1['tok'], code=0, pl='test')
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    connexion.close()

def td_coap_core_31_FAIL_TOK_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type=3, mid=coap_message1['mid'], tok=b'01', code=0)
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    connexion.close()

def td_coap_core_31_FAIL_CODE_server_emulator():
    # DONT WORK
    # waiting for receive a message CoAP/UDP/IPv4
    connexion = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    connexion.bind((IP, UDP_PORT))
    msg_recv1, addr1 = connexion.recvfrom(1024)
    #translate message
    binary_msg1 = Message(msg_recv1, CoAP)
    coap_message1 = binary_msg1.get_value()
    print('***** SERVER What we receive 1 : *****')
    print(coap_message1)
    #build response
    coap_response1 = CoAP(type=3, mid=coap_message1['mid'], tok=coap_message1['tok'], code=2.00)
    print('***** SERVER What we send 1 : *****')
    print(coap_response1)
    #send response
    msg, msg_bytes1 = coap_response1.build_message()
    connexion.sendto(msg_bytes1, (addr1[0], addr1[1]))

    connexion.close()

def main(argv):
    # Add argument with argparse to choose the interface
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--choice", choices=["ack", "noack"],
                        help="Choose if the application send acknowledgment or not")
    args = parser.parse_args()
    if args.choice == "ack":
        print("good choice")
    elif args.choice == "noack":
        print("good choice")
    else:
        # either amqp (amqp interface) or http (webserver)
        print("***********Server and Client mode with thread***********")
        _launch_sniffer('tmp/TD_COAP_CORE_23_fail.pcap')
        time.sleep(2)
        # td_coap_core_01_PASS_server_emulator()
        t1 = threading.Thread(target=td_coap_core_23_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_23_PASS_client_emulator)

        print("starting threads")
        t1.start()
        t2.start()

        print("waiting to join")
        t1.join()
        t2.join()
        time.sleep(2)
        _stop_sniffer()

        """
        #test case 1
        t1 = threading.Thread(target=td_coap_core_01_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_01_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_01_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_01_PASS_client_emulator)
        
        #test case 2
        t1 = threading.Thread(target=td_coap_core_02_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_02_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_02_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_02_PASS_client_emulator)
        
        #test case 3
        t1 = threading.Thread(target=td_coap_core_03_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_03_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_03_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_03_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_03_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_03_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_03_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_03_FAIL_client_emulator)
        
        #test case 4
        t1 = threading.Thread(target=td_coap_core_04_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_04_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_04_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_04_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_04_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_04_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_04_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_04_FAIL_client_emulator)
        
        #test case 5
        t1 = threading.Thread(target=td_coap_core_05_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_05_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_05_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_05_PASS_client_emulator)
        
        #test case 6
        t1 = threading.Thread(target=td_coap_core_06_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_06_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_06_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_06_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_06_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_06_PASS_client_emulator)
        
        #test case 7
        t1 = threading.Thread(target=td_coap_core_07_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_07_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_07_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_07_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_07_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_07_PASS_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_07_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_07_FAIL_NO_PL_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_07_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_07_FAIL_NO_OPT_client_emulator)
        
        #test case 8
        t1 = threading.Thread(target=td_coap_core_08_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_08_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_08_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_08_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_08_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_08_PASS_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_08_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_08_FAIL_NO_PL_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_08_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_08_FAIL_NO_OPT_client_emulator)
        
        #test case 9
        t1 = threading.Thread(target=td_coap_core_09_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_09_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_09_FAIL_NO_OPT_server_emulator)
        t2 = threading.Thread(target=td_coap_core_09_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_09_FAIL_NO_TOK_server_emulator)
        t2 = threading.Thread(target=td_coap_core_09_PASS_client_emulator)

        t1 = threading.Thread(target=td_coap_core_09_FAIL_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_09_PASS_client_emulator)
        
        #test case 10
        t1 = threading.Thread(target=td_coap_core_10_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_10_PASS_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_10_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_10_FAIL_TOK_0_BYTE_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_10_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_10_FAIL_TOK_9_BYTE_client_emulator)

        t1 = threading.Thread(target=td_coap_core_10_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_10_PASS_client_emulator)
        
        #test case 11
        t1 = threading.Thread(target=td_coap_core_11_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_11_PASS_client_emulator)
        
        #give none
        t1 = threading.Thread(target=td_coap_core_11_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_11_FAIL_TOK_0_BYTE_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_11_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_11_FAIL_TOK_9_BYTE_client_emulator)

        t1 = threading.Thread(target=td_coap_core_11_FAIL_NO_TOK_server_emulator)
        t2 = threading.Thread(target=td_coap_core_11_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_11_FAIL_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_11_PASS_client_emulator)
        
        #test case 12
        t1 = threading.Thread(target=td_coap_core_12_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_12_PASS_client_emulator)

        t1 = threading.Thread(target=td_coap_core_12_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_12_PASS_client_emulator)
        
        #test case 13, utiliser le serveure java de coap testting tool pour généré le pass
        t1 = threading.Thread(target=td_coap_core_13_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_13_PASS_client_emulator)

        t1 = threading.Thread(target=td_coap_core_13_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_13_FAIL_client_emulator)
        
        #test case 14
        t1 = threading.Thread(target=td_coap_core_14_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_14_PASS_client_emulator)
        
        #inconclusive
        t1 = threading.Thread(target=td_coap_core_14_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_14_FAIL_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_14_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_14_PASS_client_emulator)
        
        #test case 15
        
        #test case 16
        
        #test case 17
        t1 = threading.Thread(target=td_coap_core_17_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_17_PASS_client_emulator)

        t1 = threading.Thread(target=td_coap_core_17_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_17_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_17_FAIL_ACK_server_emulator)
        t2 = threading.Thread(target=td_coap_core_17_PASS_client_emulator)
        
        #test case 18
        t1 = threading.Thread(target=td_coap_core_18_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_18_PASS_client_emulator)
        
        #inconclusive
        t1 = threading.Thread(target=td_coap_core_18_FAIL_2_location_server_emulator)
        t2 = threading.Thread(target=td_coap_core_18_PASS_client_emulator)
        
        #inconclusive
        t1 = threading.Thread(target=td_coap_core_18_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_18_PASS_client_emulator)
        
        #test case 19
        t1 = threading.Thread(target=td_coap_core_19_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_19_PASS_client_emulator)

        t1 = threading.Thread(target=td_coap_core_19_PASS_NO_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_19_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_19_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_19_PASS_client_emulator)
        
        #inconclusive
        t1 = threading.Thread(target=td_coap_core_19_FAIL_WRONG_OPT_server_emulator)
        t2 = threading.Thread(target=td_coap_core_19_PASS_client_emulator)
        
        #test case 20
        t1 = threading.Thread(target=td_coap_core_20_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_20_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_20_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_20_FAIL_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_20_FAIL_WRONG_FORMAT_server_emulator)
        t2 = threading.Thread(target=td_coap_core_20_PASS_client_emulator)
        
        #test case 21
        t1 = threading.Thread(target=td_coap_core_21_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_21_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_21_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_21_FAIL_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_21_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_21_PASS_client_emulator)
        
        #test case 22
        t1 = threading.Thread(target=td_coap_core_22_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_22_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_22_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_22_FAIL_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_22_FAIL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_22_PASS_client_emulator)
        
        #test case 23
        t1 = threading.Thread(target=td_coap_core_23_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_23_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_23_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_23_PASS_client_emulator)
        
        #test case 31
        t1 = threading.Thread(target=td_coap_core_31_PASS_server_emulator)
        t2 = threading.Thread(target=td_coap_core_31_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_31_FAIL_PL_server_emulator)
        t2 = threading.Thread(target=td_coap_core_31_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_31_FAIL_CODE_server_emulator)
        t2 = threading.Thread(target=td_coap_core_31_PASS_client_emulator)
        
        t1 = threading.Thread(target=td_coap_core_31_FAIL_TOK_server_emulator)
        t2 = threading.Thread(target=td_coap_core_31_PASS_client_emulator)
        
        """


if __name__ == "__main__":

    main(sys.argv[1:])




#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import re, socket

from	ttproto.core.data		import *
from	ttproto.core.typecheck		import *
from	ttproto.core.exceptions		import Error
from	ttproto.core.lib.inet.meta	import *
from	ttproto.core.primitive		import BytesValue

__all__ = [
	'UInt2',
	'UInt3',
	'UInt4',
	'UInt5',
	'UInt6',
	'UInt8',
	'UInt11',
	'UInt13',
	'UInt16',
	'UInt20',
	'UInt24',
	'UInt32',
	'UInt64',
	'Bytes1',
	'Bytes2',
	'Bytes4',
	'Bytes6',
	'Bytes8',
	'IPv4Address',
	'IPv6Address',
	'IPv6Prefix',
	'ICMPv6LLAddress',
	'Eui64Address',
]

class UInt2  (metaclass = UnsignedBigEndianIntClass ( 2)):	pass
class UInt3  (metaclass = UnsignedBigEndianIntClass ( 3)):	pass
class UInt4  (metaclass = UnsignedBigEndianIntClass ( 4)):	pass
class UInt5  (metaclass = UnsignedBigEndianIntClass ( 5)):	pass
class UInt6  (metaclass = UnsignedBigEndianIntClass ( 6)):	pass
class UInt8  (metaclass = UnsignedBigEndianIntClass ( 8)):	pass
class UInt11 (metaclass = UnsignedBigEndianIntClass (11)):	pass
class UInt13 (metaclass = UnsignedBigEndianIntClass (13)):	pass
class UInt16 (metaclass = UnsignedBigEndianIntClass (16)):	pass
class UInt20 (metaclass = UnsignedBigEndianIntClass (20)):	pass
class UInt24 (metaclass = UnsignedBigEndianIntClass (24)):	pass
class UInt32 (metaclass = UnsignedBigEndianIntClass (32)):	pass
class UInt64 (metaclass = UnsignedBigEndianIntClass (64)):	pass

class Bytes1 (metaclass = FixedLengthBytesClass (1)):	pass
class Bytes2 (metaclass = FixedLengthBytesClass (2)):	pass
class Bytes4 (metaclass = FixedLengthBytesClass (4)):	pass
class Bytes6 (metaclass = FixedLengthBytesClass (6)):	pass
class Bytes8 (metaclass = FixedLengthBytesClass (8)):	pass

class IPv4Address (metaclass = FixedLengthBytesClass (4)):

	def __new__ (cls, value = None):
		# TODO: normalise the exceptions

		if isinstance (value, str):
			# convert from the textual representation
			value = socket.inet_pton (socket.AF_INET, value)

		return super().__new__(cls, value)

	def __str__ (self):
		# TODO: normalise the exceptions
		return socket.inet_ntop (socket.AF_INET, self)

	def _repr (self):
		return repr (self.__str__())

class IPv6Address (metaclass = FixedLengthBytesClass (16)):

	def __new__ (cls, value = None):
		# TODO: normalise the exceptions

		if isinstance (value, str):
			# convert from the textual representation
			value = socket.inet_pton (socket.AF_INET6, value)

		return super().__new__(cls, value)

	def __str__ (self):
		# TODO: normalise the exceptions
		return socket.inet_ntop (socket.AF_INET6, self)

	def _repr (self):
		return repr (self.__str__())



class ICMPv6LLAddress (BytesValue):

	__re_sep = re.compile ("[-.:]")

	def __new__ (cls, value = b""):
		# TODO: normalise the exceptions
		if isinstance (value, str):
			elems = re.split (cls.__re_sep, value)
#			print (value, elems)
			bins = []
			for e in elems:
				if len(e) % 2:
					e = "0" + e
				for i in range (0, len(e), 2):
					# convert from hex

					bins.append (int (e[i:i+2], 16))
			value = bytes (bins)

		if not (isinstance (value, bytes)):
			value = bytes (value)

		# add padding bytes
		return super().__new__ (cls, value + bytes ((6 - len(value)) % 8))

	def __str__ (self):
		return ":".join ("%02x" % c for c in self)

class Eui64Address (metaclass = FixedLengthBytesClass (8)):
	__reg = re.compile ("^([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})[-:.]?([0-9A-F]{2})\\Z", re.I)

	def __new__ (cls, value = None):
		if isinstance (value, str):
			mo = re.match (cls.__reg, value)
			if not mo:
				raise Error("malformed EUI-64 address")

			value = bytes ([int (v, 16) for v in mo.groups()])

		return super().__new__ (cls, value)


class IPv6Prefix (Template):
	@typecheck
	def __init__ (self, addr, length: optional(int) = None):
		Template.__init__ (self, IPv6Address)

		if length is None:
			if isinstance (addr, str) and "/" in addr:
				# address in the format "<addr>/<prefix>"
				addr, length = addr.split("/")
				length = int (length)
			else:
				# default to 64 bits
				length = 64

		assert 0 <= length <= 128
		addr = self.store_data (addr)
		prune_id = length // 8
		bitadd = length & 7
		if bitadd:
			# complex case (mask does not end at a byte boundary)
			shift = 8 - bitadd
			self.__addr = IPv6Address (addr[:prune_id] + bytes((addr[prune_id] >> shift << shift,)) + bytes(15 - prune_id))
		else:
			# simple case (mask ending at a byte boundary)
			self.__addr = IPv6Address (addr[:prune_id] + bytes(16-prune_id))

		self.__len = length

	def get_address(self):
		return self.__addr

	def get_length(self):
		return self.__len

	def _match (self, addr, difference_list = None):
		if self == IPv6Prefix (addr, self.__len):
			return True
		elif difference_list is not None:
			difference_list.append (ValueMismatch (addr, self))
		return False

	@typecheck
	def __eq__ (self, other):
		return isinstance (other, IPv6Prefix) and self.__addr == other.__addr and self.__len == other.__len

	def __ne__ (self, other):
		return not self == other

	def __str__ (self):
		return "%s/%d" % (str(self.__addr), self.__len)

	def __hash__ (self):
		return hash (hash (self.__addr) + self.__len)

	def __repr__ (self):
		return "IPv6Prefix('%s/%d')" % (str(self.__addr), self.__len)


	@typecheck
	def make_address (self, hwaddr: bytes):
		assert type (hwaddr).__name__ in ("Eui64Address", "EthernetAddress", "Ieee802154ShortAddress")
		assert self.__len <= 64


		if len (hwaddr) == 6:
			# Ethernet Address
			return IPv6Address (self.__addr[:8] + bytes((hwaddr[0] ^ 2,)) + hwaddr[1:3] + b"\xff\xfe" + hwaddr[3:])
		elif len (hwaddr) == 8:
			# EUI-64 Address
			return IPv6Address (self.__addr[:8] + bytes((hwaddr[0] ^ 2,)) + hwaddr[1:])

		else:
			assert len(hwaddr) == 2

			# short IEEE 802.15.4 Address
			return IPv6Address (self.__addr[:8] + b"\0\0\0\xff\xfe\0" + hwaddr)







#!/usr/bin/env python3

from ttproto.ts_coap.proto_specific import CoAPTestcase
from ttproto.ts_coap.proto_templates import *

class TD_COAP_CORE_01 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_01
Objective:
Perform GET transaction (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] 5.8.1,1.2,2.1,2.2,3.1

Pre-test
conditions:
•	Server offers the resource /test with resource content is not empty that handles GET with an arbitrary payload

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a GET request with:
•	Type = 0(CON)
•	Code = 1(GET)

2
Check
The request sent by the client contains:
•	Type=0 and Code=1
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Uri-Path option "test"

3
Check
Server sends response containing:
•	Code = 69(2.05 Content)
•	Message ID = CMID, Token = CTOK
•	Content-format option
•	Non-empty Paload

4
Verify
Client displays the received information
    """

    def run (self):
        self.match_coap ("client", CoAP (type="con", code="get",
                        opt = self.uri ("/test")))
        CMID = self.frame.coap["mid"]
        CTOK = self.frame.coap["tok"]

        self.next()

        if self.match_coap ("server", CoAP (
                        code = 2.05,
                        mid = CMID,
                        tok =CTOK,
                        pl = Not(b""),
                    )):
            self.match_coap ("server", CoAP (
                        opt = Opt (CoAPOptionContentFormat()),
                    ), "fail")





from ..common import *


class TD_COAP_CORE_32(CoAPTestCase):
    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='get')]

    def run(self):
        if self.match('client', CoAP(type='con', code='get', opt=self.uri('/'))):
            self.match('client', CoAP(type='con', code='get', opt=Opt(CoAPOptionOneM2MFrom())), 'fail')

        CMID = self.coap['mid']
        CTOK = self.coap['tok']

        self.next()

        if self.match('server', CoAP(code=2.05, mid=CMID, tok=CTOK, pl=Not(b'')), 'fail'):
            self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat())), 'fail')



from ttproto.tat_coap.testcases.td_coap_core_20 import TD_COAP_CORE_20


class TD_COAP_CORE_26 (TD_COAP_CORE_20):
    """
---
TD_COAP_CORE_26:
    cfg: CoAP_CFG_03
    obj: Perform GET transaction containing the Accept option (CON mode)
    pre:
        - Proxy is configured as a reverse-proxy for the server
        - Proxy’s cache is cleared
        - 'Server should provide a resource /multi-format which exists in two
           formats:'
        -   - text/plain;charset=utf-8
            - application/xml
    ref: '[1] clause 5.8.1,5.10.5,5.10.4, 8.2.2,8.2.1,10.2.2,11.2'
    seq:
        -   s: Client is requested to send a confirmable GET request to proxy
        -   c: Proxy receives the request from client & forwards it to
               server's resource
        -   c:
            - 'Forwarded request must contain:'
            -   - Type = 0 (CON)
                - Code = 1 (GET)
                - 'Option: type = Accept, value = 0 (text/plain;charset=utf-8)'
        -   c:
            - 'Server sends response containing:'
            -   - Code = 69 (2.05 content)
                - 'Option: type = Content-Format, value = 0\
                   (text/plain;charset=utf-8)'
                - Payload = Content of the requested resource in
                  text/plain;charset=utf-8 format
        -   c: Proxy forwards the response to client
        -   v: Client receives & displays the response
        -   c:
            - 'Response contains:'
            - Code = 69 (2.05 content)
            - 'Option: type = Content-Format, value = 0\
               (text/plain;charset=utf-8)'
            - Payload = Content of the requested resource in
              text/plain;charset=utf-8 format
        -   s: Client is requested to send a confirmable GET request to Proxy
        -   c: Proxy forwards the request to server
        -   c:
            - 'Sent request must contain:'
            -   - Type = 0 (CON)
                - 'Code = 1 (GET) Option: type = Accept, value = 41\
                   (application/xml)'
        -   c:
            - 'Server sends response containing:'
            -   - Code = 69 (2.05 content)
                - 'Option: type = Content-Format, value = 41\
                   (application/xml)'
                - Payload = Content of the requested resource in
                  application/xml format
        -   c: Proxy forwards the response to client
        -   v: Client receives & displays the response
        -   c:
            - 'Client displays the response received:'
            -   - Code = 69 (2.05 content)
                - 'Option: type = Content-Format, value = 41\
                   (application/xml)'
                - Payload = Content of the requested resource in
                  application/xml format
    """

    reverse_proxy = True



#!/usr/bin/env python3

from ..common import *


class TD_COAP_CORE_05 (CoAPTestCase):
    """
---
TD_COAP_CORE_05:
    cfg: CoAP_CFG_BASIC
    obj: Perform GET transaction (NON mode)
    pre: Server offers a /test resource with resource content is not empty
        that handles GET
    ref: '[COAP] 5.8.1, 5.2.3'
    seq:
    -   s:
        - 'Client is requested to send a GET request with:'
        -   - Type = 1 (NON)
            - Code = 1 (GET)
    -   c:
        - 'The request sent by the client contains:'
        -   - Type=1 and Code=1
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Uri-Path option "test"
    -   c:
        - 'Server sends response containing:'
        -   - Type = 1 (NON)
            - Code = 2.05 (Content)
            - "Server-generated Message ID (\u2794 SMID)"
            - Token = CTOK
            - Content-format option
    -   v: Client displays the received information
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        # TODO client message w/ no content format -> fail?
        return [CoAP(type='non', code='get')]

    def run (self):
        self.match ("client", CoAP (type="non", code="get",
                        opt = self.uri ("/test")))
        CTOK = self.coap["tok"]

        self.next()

        if self.match ("server", CoAP (
                        type = "non",
                        code = 2.05,
                        tok = CTOK,
                )):
            self.match ("server", CoAP (
                        opt = Opt (CoAPOptionContentFormat())
                ), "fail")





from ..common import *


class TD_COAP_CORE_29 (CoAPTestCase):
    """
---
TD_COAP_CORE_29:
    cfg: CoAP_CFG_03
    obj: Perform GET transaction with responses containing the Max-Age option
         (Reverse proxy)
    pre:
        - Proxy offers a cache
        - Proxy is configured as a reverse-proxy for the server
        - Servers resource vary in time and supports Max-Age option
        - Proxy’s cache is cleared
        - Server offers a resource /validate that varies in time, with a
          Max-Age set to 30s
    ref: '[1] clause 5.8.1,5.10.6,5.9.1.3,5.9.1.5, 8.2.2,8.2.1,10.2.2,11.2'
    seq:
        -   s: A confirmable GET request is sent to Proxy from Client
        -   c:
            - 'Proxy Sends request containing:'
            -   - Type = 0 (CON)
                - Code = 1 (GET)
        -   c:
            - 'Server sends response containing:'
            -   - Code = 69 (2.05 Content)
                - Option type = ETag
                - Option value = ETag value
                - Option type = Max-age
                - Option value
                - Not empty Payload
        -   v: Proxy forwards response to client
        -   s: A confirmable GET request is sent to proxy from Client before
               Max-Age expires
        -   c: Proxy dos not forward any request to the server
        -   c: Proxy sends response to client
        -   v:
            - 'Response contains:'
            -   - Option type = Max-age
                - Option Value = new Max-age
                - Payload cached
    """
    reverse_proxy = True

    def run(self):
        # Step 2
        self.match("client", CoAP(type="con", code="get",
                                       opt=All(
                                           Opt(CoAPOptionUriPath("validate")),
                                           NoOpt(CoAPOptionETag()),
                                       )))

        self.next_skip_ack()

        if not self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                              code=2.05,
                                              opt=Opt(CoAPOptionETag(), CoAPOptionMaxAge()),
                                              pl=Not(b""))):
            raise self.Stop()

        maxage = self.coap["opt"][CoAPOptionMaxAge]["val"]

        ts = self._frame.ts

        self.next_skip_ack(optional=True)

        while self.chain(optional=True):
            interval = self._frame.ts - ts

            if interval >= maxage:
                break

            if self.match("client", CoAP(type="con", code="get",
                                              opt=Opt(CoAPOptionUriPath("validate"))),
                               None):
                raise self.set_verdict("inconclusive",
                                      "Proxy sent a new GET request after %.1f seconds whereas Max-Age is set to %d seconds" % (
                                      interval, maxage))
                raise self.Stop()

            while self.next(optional=True):
                pass

        self.set_verdict("pass", "No further GET requests were observed within Max-Age (%d) seconds" % maxage)
